# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

import django
from django.conf import settings

sys.path.insert(0, os.path.abspath('..'))  # so that we can access the ai_django_core package
settings.configure(
    INSTALLED_APPS=[
        'django.contrib.admin',
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.messages',
        'django.contrib.staticfiles',
        'ai_django_core',
    ],
    EMAIL_BACKEND_REDIRECT_ADDRESS='',
    SECRET_KEY='ASDFjklö123456890',
)
django.setup()

from ai_django_core import __version__  # noqa: E402

# -- Project information -----------------------------------------------------

project = 'ai-django-core'
copyright = '2021, Ambient Innovation: GmbH'
author = 'Ambient Innovation: GmbH <hello@ambient-innovation.com>'
version = __version__
release = __version__

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    # 'recommonmark',  # not needed because uf m2r2
    'sphinx_rtd_theme',
    'sphinx.ext.autodoc',
    'm2r2',
]

source_suffix = ['.rst', '.md']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# html_theme = 'alabaster'
html_theme = 'sphinx_rtd_theme'
html_theme_options = {
    'display_version': False,
    'style_external_links': False,
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# Set master doc file
master_doc = 'index'



import calendar
import datetime

import pytz
from django.test import TestCase
from django.test.utils import override_settings
from freezegun import freeze_time

from ai_django_core.utils.date import date_month_delta, get_start_and_end_date_from_calendar_week, tz_today, \
    get_next_calendar_week, next_weekday, add_months, add_days, add_minutes, get_next_month, first_day_of_month, \
    get_formatted_date_str, get_time_from_seconds, datetime_format


class DateUtilTest(TestCase):
    TEST_CURRENT_YEAR = 2017

    def test_get_start_and_end_date_from_calendar_week(self):
        monday, sunday = get_start_and_end_date_from_calendar_week(2016, 52)
        self.assertEqual(monday, datetime.date(year=2016, month=12, day=26))

        monday, sunday = get_start_and_end_date_from_calendar_week(2018, 1)
        self.assertEqual(monday, datetime.date(year=2018, month=1, day=1))

        monday, sunday = get_start_and_end_date_from_calendar_week(2017, 30)
        self.assertEqual(monday, datetime.date(year=2017, month=7, day=24))

    def test_get_next_calendar_week_any_week(self):
        self.assertEqual(get_next_calendar_week(datetime.date(year=2020, month=9, day=19)), 39)

    def test_get_next_calendar_week_first(self):
        self.assertEqual(get_next_calendar_week(datetime.date(year=2020, month=1, day=1)), 2)

    def test_get_next_calendar_week_last(self):
        # We expect calendar week 1 (for 2021)
        self.assertEqual(get_next_calendar_week(datetime.date(year=2020, month=12, day=31)), 1)

    def test_next_weekday_any_weekday(self):
        self.assertEqual(next_weekday(datetime.date(year=2020, month=9, day=19), calendar.FRIDAY),
                         datetime.date(year=2020, month=9, day=25))

    def test_next_weekday_same_weekday(self):
        self.assertEqual(next_weekday(datetime.date(year=2020, month=9, day=19), calendar.SATURDAY),
                         datetime.date(year=2020, month=9, day=26))

    def test_date_month_delta(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=2, day=1)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=3, day=1)
        self.assertEqual(date_month_delta(start_date, end_date), 1)

    def test_date_month_delta_half_month(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=5, day=1)
        self.assertAlmostEqual(date_month_delta(start_date, end_date), 0.5, 1)

    def test_date_month_delta_one_and_a_half_month(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=6, day=1)
        self.assertAlmostEqual(date_month_delta(start_date, end_date), 1.5, 1)

    def test_date_month_delta_full_year(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=1, day=1)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR + 1, month=1, day=1)
        self.assertEqual(date_month_delta(start_date, end_date), 12)

    def test_date_month_delta_two_years(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=1, day=1)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR + 2, month=7, day=1)
        self.assertEqual(date_month_delta(start_date, end_date), 30)

    def test_date_month_delta_no_difference(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        self.assertEqual(date_month_delta(start_date, end_date), 0)

    def test_date_month_delta_one_day(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=16)
        self.assertEqual(date_month_delta(start_date, end_date), 1 / 30)

    def test_date_month_delta_start_greater_then_end_date(self):
        start_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=15)
        end_date = datetime.date(year=self.TEST_CURRENT_YEAR, month=4, day=14)
        with self.assertRaises(NotImplementedError):
            date_month_delta(start_date, end_date)

    @override_settings(USE_TZ=True)
    def test_tz_today_as_object_tz_active(self):
        frozen_date = datetime.datetime(year=2019, month=9, day=19, hour=10)
        with freeze_time(frozen_date):
            self.assertEqual(tz_today(), frozen_date.date())

    def test_tz_today_as_object_tz_not_active(self):
        frozen_date = datetime.datetime(year=2019, month=9, day=19, hour=10)
        with freeze_time(frozen_date):
            self.assertEqual(tz_today(), frozen_date.date())

    @override_settings(USE_TZ=True)
    def test_tz_today_as_str(self):
        frozen_date = datetime.datetime(year=2019, month=9, day=19, hour=10)
        with freeze_time(frozen_date):
            self.assertEqual(tz_today('%d.%m.%Y'), '19.09.2019')

    def test_add_months_one_month(self):
        source_date = datetime.date(year=2020, month=6, day=26)
        self.assertEqual(add_months(source_date, 1), datetime.date(year=2020, month=7, day=26))

    def test_add_months_many_months(self):
        source_date = datetime.date(year=2020, month=6, day=26)
        self.assertEqual(add_months(source_date, 10), datetime.date(year=2021, month=4, day=26))

    def test_add_months_negative_months(self):
        source_date = datetime.date(year=2020, month=6, day=26)
        self.assertEqual(add_months(source_date, -2), datetime.date(year=2020, month=4, day=26))

    def test_add_days_one_day(self):
        source_date = datetime.date(year=2020, month=6, day=26)
        self.assertEqual(add_days(source_date, 1), datetime.date(year=2020, month=6, day=27))

    def test_add_days_many_days(self):
        source_date = datetime.date(year=2020, month=6, day=26)
        self.assertEqual(add_days(source_date, 10), datetime.date(year=2020, month=7, day=6))

    def test_add_days_negative_days(self):
        source_date = datetime.date(year=2020, month=6, day=26)
        self.assertEqual(add_days(source_date, -2), datetime.date(year=2020, month=6, day=24))

    def test_add_minutes_one_minute(self):
        source_datetime = datetime.datetime(year=2020, month=6, day=26, hour=8, minute=0)
        self.assertEqual(add_minutes(source_datetime, 1),
                         datetime.datetime(year=2020, month=6, day=26, hour=8, minute=1))

    def test_add_minutes_many_minutes(self):
        source_datetime = datetime.datetime(year=2020, month=6, day=26, hour=8, minute=0)
        self.assertEqual(add_minutes(source_datetime, 10),
                         datetime.datetime(year=2020, month=6, day=26, hour=8, minute=10))

    def test_add_minutes_negative_minutes(self):
        source_datetime = datetime.datetime(year=2020, month=6, day=26, hour=8, minute=0)
        self.assertEqual(add_minutes(source_datetime, -2),
                         datetime.datetime(year=2020, month=6, day=26, hour=7, minute=58))

    @freeze_time('2020-06-26')
    def test_get_next_month_regular(self):
        self.assertEqual(get_next_month(), datetime.date(year=2020, month=7, day=26))

    def test_first_day_of_month_regular(self):
        source_date = datetime.date(year=2020, month=6, day=26)
        self.assertEqual(first_day_of_month(source_date), datetime.date(year=2020, month=6, day=1))

    def test_get_formatted_date_str_regular(self):
        source_date = datetime.date(year=2020, month=6, day=26)
        self.assertEqual(get_formatted_date_str(source_date), '26.06.2020')

    def test_get_time_from_seconds_one_hour(self):
        self.assertEqual(get_time_from_seconds(3600), '01:00:00')

    def test_get_time_from_seconds_one_minute(self):
        self.assertEqual(get_time_from_seconds(60), '00:01:00')

    def test_get_time_from_seconds_big_hours(self):
        self.assertEqual(get_time_from_seconds(3600 * 99), '99:00:00')

    def test_get_time_from_seconds_huge_hours(self):
        self.assertEqual(get_time_from_seconds(3600 * 1000), '1000:00:00')

    def test_get_time_from_seconds_negative_seconds(self):
        with self.assertRaises(ValueError):
            get_time_from_seconds(-1)

    @override_settings(TIME_ZONE='UTC')
    def test_datetime_format_regular(self):
        source_date = datetime.datetime(year=2020, month=6, day=26, hour=8, tzinfo=pytz.UTC)
        self.assertEqual(datetime_format(source_date, '%d.%m.%Y %H:%M'), '26.06.2020 08:00')

    @override_settings(TIME_ZONE='Europe/Cologne')
    def test_datetime_format_wrong_timezone(self):
        source_date = datetime.datetime(year=2020, month=6, day=26, hour=8)
        self.assertEqual(datetime_format(source_date, '%d.%m.%Y %H:%M'), '26.06.2020 08:00')

    @override_settings(TIME_ZONE='Europe/Berlin')
    def test_datetime_format_different_timezone(self):
        source_date = datetime.datetime(year=2020, month=6, day=26, hour=8, tzinfo=pytz.UTC)
        self.assertEqual(datetime_format(source_date, '%d.%m.%Y %H:%M'), '26.06.2020 10:00')



from rest_framework import serializers

from testapp.models import MySingleSignalModel


class MySingleSignalModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = MySingleSignalModel
        fields = [
            'id',
            'value',
        ]



from rest_framework.routers import DefaultRouter

from testapp.api.views import MySingleSignalModelViewSet

model_router = DefaultRouter()
model_router.register(r'my-single-signal-model', MySingleSignalModelViewSet, basename='my-single-signal-model')



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['ai_django_core',
 'ai_django_core.admin',
 'ai_django_core.admin.model_admins',
 'ai_django_core.admin.views',
 'ai_django_core.drf',
 'ai_django_core.graphql',
 'ai_django_core.graphql.forms',
 'ai_django_core.graphql.schemes',
 'ai_django_core.graphql.tests',
 'ai_django_core.mail',
 'ai_django_core.mail.backends',
 'ai_django_core.mail.services',
 'ai_django_core.middleware',
 'ai_django_core.mixins',
 'ai_django_core.services',
 'ai_django_core.templatetags',
 'ai_django_core.tests',
 'ai_django_core.utils',
 'ai_django_core.view_layer']

package_data = \
{'': ['*'],
 'ai_django_core': ['locale/*',
                    'locale\\de/*',
                    'locale\\de\\LC_MESSAGES/*',
                    'pipeline/*',
                    'pipeline\\services/*']}

install_requires = \
['Django>=2.2.24',
 'bleach>=1.4.3',
 'python-dateutil>=2.5.3',
 'html2text>=2020.1.16']

extras_require = \
{'dev': ['freezegun',
         'pytest-cov',
         'pytest-django',
         'pytest-mock',
         'gevent',
         'sphinx',
         'recommonmark',
         'sphinx-rtd-theme',
         'm2r2'],
 'docs': ['recommonmark', 'sphinx-rtd-theme', 'm2r2'],
 'drf': ['djangorestframework>=3.8.2'],
 'graphql': ['graphene-django>=2.2.0', 'django-graphql-jwt>=0.2.1'],
 'view-layer': ['django-crispy-forms>=1.4.0']}

setup(name='ai-django-core',
      version='5.8.0',
      description='Ambient toolbox - Lots of helper functions and useful widgets',
      author=None,
      author_email='"Ambient Innovation: GmbH" <hello@ambient-innovation.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
     )



import graphene
from django.utils.decorators import method_decorator
from graphql import GraphQLError
from graphql_jwt.decorators import login_required


class DeleteMutation(graphene.ClientIDMutation):
    """
    Provides a mutation for handling common delete cases. Exposes methods for custom validation and queryset filtering.
    """
    success = graphene.Boolean()
    model = None

    class Meta:
        abstract = True

    class Input:
        id = graphene.ID()

    @classmethod
    def __init_subclass_with_meta__(cls, resolver=None, output=None, arguments=None, _meta=None, model=None, **options):
        if not model:
            raise AttributeError('DeleteMutation needs a valid model to be set.')
        super().__init_subclass_with_meta__(resolver, output, arguments, _meta, **options)
        cls.model = model

    @classmethod
    def validate(cls, request):
        """
        Feel free to put any kind of custom validation rules here
        """
        return True

    @classmethod
    def get_queryset(cls, request):
        """
        Defines the queryset on which the object with the given ID can be chosen
        """
        return cls.model.objects.all()

    @classmethod
    def mutate_and_get_payload(cls, root, info, **input_data):
        """
        Ensure custom validation, fetch object and delete it afterwards.
        """
        if not cls.validate(info.context):
            raise GraphQLError('Delete method not allowed.')

        # Get object id
        object_id = int(input_data.get('id', None))

        # Find and delete object
        obj = cls.get_queryset(info.context).get(pk=object_id)
        obj.delete()

        # Return success
        return DeleteMutation()


@method_decorator(login_required, name='mutate_and_get_payload')
class LoginRequiredDeleteMutation(DeleteMutation):
    """
    Deletes an object from the database.
    Ensures user is authenticated with GraphQL-JWT
    """
    class Meta:
        abstract = True
